// $Id: $

#ifndef __MODULE_H__
#define __MODULE_H__


#define MODULE_API_MAJOR 0
#define MODULE_API_MINOR 3
#define MODULE_API_PATCH 0



#define MOD_ERR_MSG -1
#define MOD_ERR_OK 0
#define MOD_ERR_STOP 1
#define MOD_ERR_CRIT 2
#define MOD_ERR_FATAL 3
#define MOD_ERR_EXISTS 4
#define MOD_ERR_NOFILE 5
#define MOD_ERR_IO 6
#define MOD_ERR_API 7
#define MOD_ERR_ABI 8
#define MOD_ERR_MEMORY 9
#define MOD_ERR_UNLOAD 10
#define MOD_ERR_SYNTAX 11
#define MOD_ERR_UNSAFE 12
#define MOD_ERR_NOMOD  13
#define MOD_ERR_SQL    14




#define MOD_CONT MOD_ERR_OK
#define MOD_STOP MOD_ERR_STOP
#define MOD_CRIT MOD_ERR_CRIT
#define MOD_FATAL MOD_ERR_FATAL
#define MOD_MEMORY MOD_ERR_MEMORY


char mod_err_msg[512];


#define DelModule(x) module_close(x)


#define throwModMsg(msg, ...) \
		snprintf(mod_err_msg, sizeof(mod_err_msg), msg, __VA_ARGS__); \
		return MOD_ERR_MSG; \


#define addDependancy(mod) do {         \
					\
		if (FindModule(mod))    \
			break;		\
					\
		snprintf(mod_err_msg,sizeof(mod_err_msg),      \
			"Missing module depandancy (%s)",mod); \
					\
		return MOD_ERR_MSG;     \
					\
		} while (0);


enum MOD_TYPES {

	THIRD     = 0,
	PROTOCOL  = 1,
	CORE      = 2,
	SUPPORTED = 3
};


int nomodules;

typedef void (closefnct)();

typedef struct module_ {

    time_t age;            //Time we were loaded

    char name[50];         //name of module (Modules are allowed to specify there own name)
                           //however if they dont the name will be set the same as filename

    char file[255];         //We have to store the filename because module_open passes off paths.

    char version[75];     //Version information in a string - This has no impact on weather or

    char author[25];       //Who wrote the module

    void* handle;

    struct module_ *prev,*next;  //Next/Previous modules in our list

    int type;                    //Type of module (this is depreciated a bit as the core
                                 //knows which modules its loading as s'posed to off the
                                 //what modules tell us.

    int comp_version;


    //version information :)
    long abi;                //BINARY INTERFACE version

    int loaded;              //is this module loaded yet?

    int clssym;              //do we have a close symbols for the close function? 1 = yes
              //everything else = no - this will also
              //describe if our modules are safe to unload - as we dont
              //want to unload modules that dont cleanup on unload


    closefnct* close;        //ptr to our close function.


} Module;


Module* currentModule;

typedef int initfnct();

dlink_list  modules;

/***********************************/
/**
 * Module Que Structures
 */


typedef struct moduleq_entry {
	char *name;

	int action;

	struct moduleq_entry *prev;
	struct moduleq_entry *next;

} ModuleQEntry;

dlink_list moduleque;


/***********************************/



int load_protocol();
int load_core();
int load_modules();
int load_client_modules();

int rehash_core_modules();
int rehash_client_modules();
int unload_all_modules(int);

/*************************************/


int module_open(char *filename);
char *find_module_dir(char *module);
int module_exists (char * fileName);

Module* FindModule  (char *);

void    module_free (Module *);

int addModuleQue(char *,int);
int runModuleQue();

int RegisterModule(char*, char* ,char *, int, int ,int);
int CheckVer();


#endif /*__MODULE_H__*/


