#!/bin/sh
#
# Build version string and increment Services build number.
# coded by twitch and based very loosly on Anope's version
# script
#
# $Id: version.sh 1165 2008-08-02 10:12:51Z jer $

# Grab version information from the version control file.

REV="0"
UNAME=`uname -rsnm`

if [ -f version.log ] ; then
  CTRL="version.log"
  TODIR="include/"
else
  CTRL="../version.log"
  TODIR=""
fi

if [ -f $CTRL ] ; then
	. $CTRL
else
	echo "Error: Unable to find control file: $CTRL"
	exit 0
fi


if [ -f ${TODIR}version.h ] ; then
	BUILD=`fgrep '#define BUILD' ${TODIR}version.h | sed -n 's/^#define BUILD.*\"\(.*\)\"/\1/p'`
	BUILD=`expr $BUILD + 1 2>/dev/null`
else
	BUILD=1
fi


for dir in `echo -n $PATH | sed 's/:/ /g'`; do
    if [ -x "${dir}/svnversion" ]; then
        if [ -d ".svn" ]; then
            REV=`svnversion | sed 's/^\([0-9]{1,9}*\)M*$/\1/'`

            if test "X$REV" = "Xexported"; then
                REV="0"
            fi
        else
            REV="0"
        fi
    fi
done

VERSION="${VERSION_MAJOR}.${VERSION_MINOR}.${VERSION_PATCH}${VERSION_EXTRA} (r${REV})"
VERSIONDOTTED="${VERSION_MAJOR}.${VERSION_MINOR}.${VERSION_PATCH}${VERSION_EXTRA}.${REV}"
CDATE=`date`

cat >${TODIR}version.h <<EOF
/* Version information for Omega Server.
 *
 * (C) 2008 Omega Team
 *
 *
 * Please read COPYING and CREDITS for further details.
 *
 * Based on loosly on version.sh from Anope 1.7.x by Anope Team
 *
 * This file is auto-generated by version.sh
 *  $Id: version.sh 1165 2008-08-02 10:12:51Z jer $
 *
 */

#ifndef __VERSION_H__
#define __VERSION_H__

#define VERSION_MAJOR	$VERSION_MAJOR
#define VERSION_MINOR	$VERSION_MINOR
#define VERSION_PATCH	$VERSION_PATCH
#define VERSION_EXTRA	"$VERSION_EXTRA"
#define VERSION_BUILD	"$REV" 

#define BUILD	"$BUILD"
#define VERSION_STRING "$VERSION"
#define VERSION_STRING_DOTTED "$VERSIONDOTTED"

#define SYSUNAME    "$UNAME"

#define COMPILE_DATE "$CDATE" 

#ifdef DEBUG_COMMANDS
# define VER_DEBUG "D"
#else
# define VER_DEBUG
#endif

#endif //__VERSION_H__
EOF

exit 0


