#include "stdinc.h"
#include "server.h"
#include "extern.h"

#define NAME "g_core"
#define VERSION "$Id: g_core.c 1268 2008-08-20 08:39:36Z twitch $"
#define AUTHOR "Twitch"


static void cmd_shutdown  (User*, char *);
static void cmd_modules   (User*, char *);
static void cmd_access    (User*, char *);
static void cmd_rehash    (User*, char *);
static void cmd_restart   (User*, char *);
static void cmd_update     (User*,char *);

static void help_gcore     (User*);


static void help_shutdown_ext (User*);
static void help_restart_ext  (User*);
static void help_rehash_ext   (User*);
static void help_access_ext   (User*);
static void help_module_ext   (User*);


extern void do_restart(void);

int Module_Init()
{
	int ch; 
     
      Log("[%s] registering module",NAME);

      if (RegisterModule(NAME,AUTHOR,VERSION,0,4,0) != 0)
		return MOD_ERR_ABI;

      addDependancy ("m_guardian");

      ch = AddCmd(s_Guardian, "MODULE", ACC_FLAG_MODULE, cmd_modules);
      AddHelpExt(ch,s_Guardian,help_module_ext);

      ch = AddCmd(s_Guardian, "SHUTDOWN", ACC_FLAG_DIE, cmd_shutdown);
      AddHelpExt(ch,s_Guardian,help_shutdown_ext);

      ch = AddCmd(s_Guardian, "REHASH", ACC_FLAG_DIE, cmd_rehash);
      AddHelpExt(ch,s_Guardian,help_rehash_ext);

      ch = AddCmd(s_Guardian, "RESTART", ACC_FLAG_DIE, cmd_restart);
      AddHelpExt(ch,s_Guardian,help_restart_ext);

      ch = AddCmd(s_Guardian, "ACCESS", ACC_OPER, cmd_access);
      AddHelpExt(ch,s_Guardian,help_access_ext);

      AddHelp(s_Guardian,help_gcore);

      return MOD_CONT;


}


void Module_Close()
{

	Log("[%s] unloading module",NAME);

	DelCmd(s_Guardian,"MODULE", cmd_modules);
	DelCmd(s_Guardian,"ACCESS", cmd_access);
	DelCmd(s_Guardian,"SHUTDOWN", cmd_shutdown);
	DelCmd(s_Guardian,"REHASH", cmd_rehash);
	DelCmd(s_Guardian,"RESTART", cmd_restart);

	DelHelp(s_Guardian,help_gcore);

	Log("[%s] unload successful all commands deleted",NAME);

	return;

}

/***************************************************/

static void cmd_rehash(User* u, char* av)
{
	ircd_wallop(s_Guardian->nick,"%s is rehashing the server",u->nick);
	Rehash(0);
	return;
}

/****************************************************/


//you know its safe to say we can destroy this buffer.
static void cmd_modules(User* u, char* av)
{
	Module* m;
	dlink_node* dl;

	int n, status;
	int i = 0;
	char* params[512];
	char path[255];

	n = generictoken(' ',512,av,params);

	if ((!av) || (strcasecmp(params[0],"LIST")==0))
	{

	      sendto_one (s_Guardian, u, "Current Module List:");
             DLINK_FOREACH (dl, modules.head)
	      {
			m = dl->data;
                     if ((m->name) && (m->type != 2) && (m->type != 1))  {
				i++;
				sendto_one (s_Guardian, u, "\002%s\002 [%s]",
						 m->name, m->version);
			 }
              }

              sendto_one(s_Guardian, u, "\002%d\002 modules loaded.",i);

             return;
	}

	if (!params[1])
	{
		sendto_one(s_Guardian,u,"Invalid Syntax: This command requires a parameter");
		return;
	}

	if (strcasecmp(params[1],"m_guardian")==0)
	{
		sendto_one(s_Guardian,u,"You can not perform any actions on this module");
		return;
	}

	if (strcasecmp(params[0],"UNLOAD")==0)
	{

		if (params[1])
		{
			sendto_one(s_Guardian,u,"Unloading module %s",params[1]);
			addModuleQue(params[1],1);
		}
		else
			sendto_one (s_Guardian,u,"Invalid Paramaters");
		return;
	}

	if (strcasecmp(params[0],"LOAD")==0)
	{
		if (n == 1)
		{
			sendto_one(s_Guardian,u,"Invalid Parameter");
			return;
		}
		if (params[1])
		{
			sprintf(path,"%s/%s",find_module_dir(params[1]),params[1]);
			sendto_one(s_Guardian,u,"Loading module %s",params[1]);
			ircd_wallop(s_Guardian->nick,"%s has loaded module %s",u->nick,params[1]);
			addModuleQue(path,0);

		}
		else
			sendto_one (s_Guardian,u,"Invalid Paramaters");

		return;
	}

	if (strcasecmp(params[0],"RELOAD")==0)
	{
		if (!params[1])
		{
			sendto_one(s_Guardian,u,"Invalid paramater - you must specify a MODULE to reload");
		}
		
		if (addModuleQue(params[1],1) != MOD_ERR_OK)
		{
			sendto_one (s_Guardian,u,"Unable to reload module %s - module doesnt exsist",params[1]);
			return;
		}
		sprintf(path,"%s/%s",find_module_dir(params[1]),params[1]);
		addModuleQue(path,0);
		sendto_one (s_Guardian,u,"Module %s has been reloaded",params[1]);
		sendto_logchan("%s has reloaded module [%s]",u->nick,params[1]);
		return;
	}
	return;
}



/***************************************************/


static void cmd_shutdown(User *u, char *av)
{
	ircd_wallop(s_Guardian->nick,"Shutdown CMD issued by %s (%s@%s)",u->nick,u->user,u->host);
	alog(0,"Shutdown CMD issued from the server by %s (%s@%s)",u->nick,u->user,u->host);
	Exit(0);
	return;

}



/**********************************************/

static void cmd_access(User* u,char* av)
{
	dlink_node *dl,*hdl;
	Access* ca;
	char* host;
	char* params[512];
	int n;

	n = generictoken(' ',512,av,params);

	if ((!av) || (strcasecmp(params[0],"LIST")==0))
	{

		DLINK_FOREACH(dl,accesslist.head)
		{
			 ca = dl->data;

			DLINK_FOREACH(hdl,ca->hosts.head)
			{
				host = hdl->data;
				sendto_one(s_Guardian,u,"%s %s %lu",ca->name,host,ca->flags);

			}
		}
		return;

	}
}





/************************************************/


static void cmd_restart(User* u,char* av)
{
	do_restart();
	return;
}

/************************************************/
/**
 * Help functions
 */

static void help_gcore (User* u)
{
		sendto_one(s_Guardian,u,"%s %-20s","ACCESS",   " - Manage the internal access list");
		sendto_one(s_Guardian,u,"%s %-20s","SHUTDOWN", " - Clean up then shutdown the server.");
		sendto_one(s_Guardian,u,"%s %-20s","MODULE",   " - Manage loaded modules");
		sendto_one(s_Guardian,u,"%s %-20s","RESTART",  " - Restart the server");
		sendto_one(s_Guardian,u,"%s %-20s","REHASH",   " - Reload all configuartion files");
              sendto_one(s_Guardian,u,"%s %-20s","UPDATE",   " - Force all modules to save there databases");
	return;
}

static void help_shutdown_ext (User* u)
{
	sendto_one(s_Guardian,u,"Syntax: \002SHUTDOWN\002");
	sendto_one(s_Guardian,u,"");
	sendto_one(s_Guardian,u,"     Cuases server to cleanup and shutdown.");
	sendto_one(s_Guardian,u,"");
	return;
}

static void help_restart_ext (User* u)
{
	sendto_one(s_Guardian,u,"Syntax: \002RESART\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"     Cuases server to cleanup and restart its services.");
	sendto_one(s_Guardian,u," ");
	return;
}

static void help_rehash_ext (User* u)
{
	sendto_one(s_Guardian,u,"Syntax: \002REHASH\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"     Reload all configuration files");
	sendto_one(s_Guardian,u," ");
	return;
}

static void help_access_ext (User* u)
{
	sendto_one(s_Guardian,u,"Syntax: \002ACCESS [OPTION]\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"\002ACCESS LIST\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"     List the entries in the current internal access list");
	sendto_one(s_Guardian,u," ");
	return;
}

static void help_module_ext (User* u)
{
	sendto_one(s_Guardian,u,"Syntax: \002MODULE [OPTION]\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"\002MODULE LIST\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"     List all currently loaded modules");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"\002MODULE LOAD [FILENAME]\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"     Load the module named FILENAME into the server");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"\002MODULE UNLOAD [FILENAME]\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u," Unload the module named FILENAME from the server");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"\002MODULE RELOAD [FILENAME]\002");
	sendto_one(s_Guardian,u," ");
	sendto_one(s_Guardian,u,"     Unload then reload the module named FILENAME");
	sendto_one(s_Guardian,u," ");
	return;
}
