#include "stdinc.h"
#include "server.h"

#define NAME	"m_omegastats"
#define	AUTHOR	"Jer"
#define VERSION	"$Revision: 1374 $"

#define HTTP_ADDR	"stats.ssnet.ca"
#define HTTP_SERV	"stats.ssnet.ca"
#define HTTP_PORT	80

Socket	*statsock;

static void		sock_handle	(Socket *, int);
static void		sock_read	(Socket *);
static void		sock_write	(Socket *);

void	evh_omegastats	(int, char **);

int Module_Init ()
{
	RegisterModule (NAME, AUTHOR, VERSION, 0, 4, 0);

	AddTimedEvent	("Omega Statistical Post", evh_omegastats, 900);

	// Always run this at load time
	evh_omegastats (0, NULL);

	return MOD_CONT;
}

void Module_Close ()
{
	dlink_node	*dl;
	DelTimedEvent	("Omega Statistical Post");

	// Keep this for a bit
	DelTimedEvent	("Omega Statistical Gather");

	if ((statsock = find_socket ("statsock")))
	{
		close (statsock->sd);
		dl	= dlink_find_delete (statsock, &sockets);
		dlink_free (dl);
		free (statsock);
	}

	return;
}

static void sock_handle	(Socket *s, int e)
{
	if (!(statsock = find_socket ("statsock")))
		return;

	if (s->sd != statsock->sd)
		return;

	switch (e)
	{
		case EVENT_WRITE:
			sock_write (s);
			break;
		case EVENT_READ:
			sock_read (s);
			break;

		default:
			break;
	}

	return;
}

static void	sock_read	(Socket *s)
{
	char	*msg;

	while (s->fullLines > 0)
	{
		msg	= GetLine (s);

		if (msg)
			free (msg);
	}

	return;
}

static void sock_write	(Socket *s)
{
	int		ret;
	char	content[2048];
	char	headers[2048];

	memset (content, '\0', sizeof(content));
	memset (headers, '\0', sizeof(headers));

	snprintf (content, 2048, "servername=%s&uplink=%s&version=%s&protocol=%s&uplinkname=%s", CfgSettings.servername, CfgSettings.uplink,
				VERSION_STRING_DOTTED, CfgSettings.protocol, (Uplink)? Uplink->name : "not%20connected");
	snprintf (headers, 2048, "User-Agent: Omega/%s m_omegastats.c/%s\nHost: %s\nContent-Length: %d\nContent-Type: application/x-www-form-urlencoded", VERSION_STRING_DOTTED, VERSION, HTTP_SERV, strlen(content));
#if 0
	printf ("Sending:\nPOST /omega/omega.php HTTP/1.1\n%s\n\n%s\r\n\r\n", headers, content);
#endif
	ret = sendto_socket (s, "POST /omega/omega.php HTTP/1.1\n%s\n\n%s\r\n\r\n", headers, content);

	if (ret == 0)
	{
		s->flags	&= ~SOCK_CONNECTING;
		s->flags	&= ~SOCK_WRITE;
		s->flags	|= SOCK_READ;


		return;
	}

	return;
}

void evh_omegastats	(int argc, char **argv)
{
	int					ret;
	struct sockaddr_in	sa;
	dlink_node			*dl;
	char				vhost[63];

	if (!(statsock = find_socket ("statsock")))
	{
		statsock	= new_socket ();
		statsock->flags	|= SOCK_CONNECTING|SOCK_WRITE;

		add_sock_handlers(statsock, (void *) sock_handle, NULL);

		strlcpy (statsock->name, "statsock", sizeof(statsock->name));

		memset (vhost, '\0', sizeof(vhost));

		ret = Connect (statsock, HTTP_ADDR, vhost, HTTP_PORT);

		if (!ret)
		{
			dl	= dlink_find_delete (statsock, &sockets);
			dlink_free (dl);
			free (statsock);
			return;
		}

		return;
	}

	return;
}

