#include "stdinc.h"
#include "server.h"

#define NAME "m_scan"
#define VERSION "Omega threat detection module v0.1"
#define AUTHOR "Twitch"


struct scanque_ {
	User* user;
	struct scanque_ *next,*prev;
};

dlink_list ScanQ;
dlink_list VersionQ;

#if 0
static void scan_nick(Link*,int,char**);
#endif
static void scan_nick (int, User *);

void scan_que();
void version_que();
void init_events();

int HandleResult(User* u, int score);
void write_bl_log(User* u, int score);

static void cmd_version   (User *, int, char **);
static void cmd_quit      (User *, int, char **);
static void cmd_scanstats (User *,char *);

int has_q = 0; //do we have a scanq to parse?

FILE *fp;


float avg_range = 15.525;
float nick_range_val = 0;
int nick_count = 0;


//map for old style checking
//this contains our point value for various
//character

int OldCharMap[128] = {
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Horzontal Tab */
    0, 0, 0,                    /* Line Feed, Unused, Unused */
    0, 0, 0,                    /* Carriage Return, Unused, Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Space */
    0, 0, 0,                    /* ! " #  */
    0, 0, 0,                    /* $ % &  */
    0, 0, 0,                    /* ! ( )  */
    0, 0, 0,                    /* * + ,  */
    0, 0, 0,                    /* - . /  */
    1, 1,                       /* 0 1 */
    1, 1,                       /* 2 3 */
    1, 1,                       /* 4 5 */
    1, 1,                       /* 6 7 */
    1, 1,                       /* 8 9 */
    0, 0,                       /* : ; */
    0, 0, 0,                    /* < = > */
    0, 0,                       /* ? @ */
    -1, -1, -1,                 /* A B C */
    -1, -1, -1,                 /* D E F */
    -1, -1, -1,                 /* G H I */
    -1, -1, -1,                 /* J K L */
    -1, -1, -1,                 /* M N O */
    -1, -1, -1,                 /* P Q R */
    -1, -1, -1,                 /* S T U */
    -1, -1, -1,                 /* V W X */
    -1, -1,                     /* X Z   */
    0, 0, 0,                    /* [ \ ] */
    0, 0, 0,                    /* ^ _ ` */
    -1, -1, -1,                 /* a b c */
    -1, -1, -1,                 /* d e f */
    -1, -1, -1,                 /* g h i */
    -1, -1, -1,                 /* j k l */
    -1, -1, -1,                 /* m n o */
    -1, -1, -1,                 /* p q r */
    -1, -1, -1,                 /* s t u */
    -1, -1, -1,                 /* v w x */
    -1, -1,                     /* y z */
    0, 0, 0,                    /* { | } */
    0, 0                        /* ~  */
};



int CharMap[128] = {
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Horzontal Tab */
    0, 0, 0,                    /* Line Feed, Unused, Unused */
    0, 0, 0,                    /* Carriage Return, Unused, Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Space */
    0, 0, 0,                    /* ! " #  */
    0, 0, 0,                    /* $ % &  */
    0, 0, 0,                    /* ! ( )  */
    0, 0, 0,                    /* * + ,  */
    0, 0, 0,                    /* - . /  */
    0, 0,                       /* 0 1 */
    0, 0,                       /* 2 3 */
    0, 0,                       /* 4 5 */
    0, 0,                       /* 6 7 */
    0, 0,                       /* 8 9 */
    0, 0,                       /* : ; */
    0, 0, 0,                    /* < = > */
    0, 0,                       /* ? @ */
    1, 0, 1,                    /* A B C */
    2, 1, 1,                    /* D E F */
    2, 2, 1,                    /* G H I */
    2, 2, 1,                    /* J K L */
    1, 1, 1,                    /* M N O */
    2, 2, 1,                    /* P Q R */
    1, 1, 1,                    /* S T U */
    3, 2, 3,                    /* V W X */
    3, 3,                       /* X Z   */
    0, 0, 0,                    /* [ \ ] */
    0, 0, 0,                    /* ^ _ ` */
    1, 2, 1,                    /* a b c */
    0, 1, 1,                    /* d e f */
    2, 2, 1,                    /* g h i */
    3, 2, 1,                    /* j k l */
    1, 1, 1,                    /* m n o */
    2, 3, 1,                    /* p q r */
    1, 1, 1,                    /* s t u */
    3, 2, 3,                    /* v w x */
    3, 3,                       /* y z */
    0, 0, 0,                    /* { | } */
    0, 0                        /* ~  */
};

int SpecialCharMap[128] = {
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Horzontal Tab */
    0, 0, 0,                    /* Line Feed, Unused, Unused */
    0, 0, 0,                    /* Carriage Return, Unused, Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused */
    0, 0, 0,                    /* Unused, Unused, Space */
    1, 1, 1,                    /* ! " #  */
    1, 1, 1,                    /* $ % &  */
    1, 1, 1,                    /* ! ( )  */
    1, 1, 1,                    /* * + ,  */
    1, 1, 1,                    /* - . /  */
    0, 0,                       /* 0 1 */
    0, 0,                       /* 2 3 */
    0, 0,                       /* 4 5 */
    0, 0,                       /* 6 7 */
    0, 0,                       /* 8 9 */
    1, 1,                       /* : ; */
    1, 1, 1,                    /* < = > */
    1, 1,                       /* ? @ */
    0, 0, 0,                    /* A B C */
    0, 0, 0,                    /* D E F */
    0, 0, 0,                    /* G H I */
    0, 0, 0,                    /* J K L */
    0, 0, 0,                    /* M N O */
    0, 0, 0,                    /* P Q R */
    0, 0, 0,                    /* S T U */
    0, 0, 0,                    /* V W X */
    0, 0,                       /* X Z   */
    1, 1, 1,                    /* [ \ ] */
    1, 1, 1,                    /* ^ _ ` */
    0, 0, 1,                    /* a b c */
    0, 0, 1,                    /* d e f */
    0, 0, 1,                    /* g h i */
    0, 0, 1,                    /* j k l */
    0, 0, 1,                    /* m n o */
    0, 0, 1,                    /* p q r */
    0, 0, 1,                    /* s t u */
    0, 0, 0,                    /* v w x */
    0, 0,                       /* y z */
    1, 1, 1,                    /* { | } */
    1, 1                        /* ~  */
};


int Module_Init()
{
	char path[255];

	int status = 0;
        status = RegisterModule(NAME,AUTHOR,VERSION,0,4,1);

	/*
	if (status > 0)
		 throwModuleErr("Unable to register module (%d)",status);
	*/


	AddEvent ("USERCONN", scan_nick);
	AddEvent ("BURST", init_events);

        AddUserCmd("NOTICE", cmd_version);

	AddTimedEvent ("SCANQ", scan_que, 30);
	AddTimedEvent ("VersionQueue", version_que, 30);

	sprintf(path,"%s%s%s",DPATH,LPATH,"scan.log");
	fp = fopen(path,"a+");

	memset(path,0,sizeof(path));


	AddCmd(s_Guardian,"SCANSTATS",ACC_FLAG_DEBUG,cmd_scanstats);


	return 0;
}


int Module_Close()
{


	DelUserCmd("NOTICE", cmd_version);
	DelEvent ("USERCONN", scan_nick);

	DelCmd(s_Guardian,"SCANSTATS", cmd_scanstats);




	return;
}

void init_events ()
{
	AddTimedEvent ("SCANQ", scan_que, 30);
	AddTimedEvent ("versionQueue", version_que, 30);
}


/************************************************/
/**
 * We need to be very VERY careful as to not
 * destroy the string or else we fail at life
 * so since we are called AFTER the protocol
 * handles it, use the user struct for the
 * given nick.
 */

static void scan_nick (int argc, User *u)
{

        char *tmp = NULL;

	dlink_node	*dl;
	struct scanque_	*sq;

         int score  = 0;
	 int score2 = 0;

	 int special_char = 0;
	 int misc_checks  = 0;
	 int cmd_check    = 0;


	 float total = 0.0;

         int len = 0;                   //might as well count the len since were looping.
	 float average = 0;
	 float alloted_prec = 0;  
	 float percent = 0;

	if (MyConnect(u) || u->oper || u->service) 
	{
		if (cmd_check == 0)
			return;
	}

	if (argc > 1)
	{
		sendto_logchan("Performing Scan on %s",u->nick);
		cmd_check = 1;
	}


	/**
	 ** Since I want to catch versions early on, send it
	 */

       if (sync_state == 3)
       	send_line(":%s PRIVMSG %s :\1VERSION\1",s_Guardian->nick,u->nick);

	else
	{
		sq	= (struct scanque_ *) malloc (sizeof(struct scanque_));
		dl	= dlink_create ();

		sq->user	= u;

		dlink_add_tail (sq, dl, &VersionQ);
	}

    /**
     * First off start scanning for old style attackers
     * 5 - 6 letter randon alphanumeric nicks and
     * realnames, were the nick/realname have
     * more numbers then letters...
     * Note: We dont care weather or not its random
     * or purposefull - a1111 loks the same as z123d3
     *
     *  a1234!ident@host.tld:a2345 <- will get a score of 6
     *
     *  a1234!ident@host.tld:Im Just a User <-- Will get a score of -8
     *                                          and will not match our
     *                                          algorythm.
     *  Alpha = -1
     *  Numerics = +1
     *  All else ignored
     *
     *
     */


	tmp = u->nick;

     //nick
     do
     {

             score +=  OldCharMap[(int) *tmp];  
             score2 +=  CharMap[(int) *tmp];
	     special_char += SpecialCharMap[(int) *tmp];


	      *tmp++;
             len++;

     } while (*tmp);


     if (len == 5 || len == 6)
            score++;

	 //realname
	 tmp = u->realname;
	 
	 do
	 {

              score +=  OldCharMap[(int) *tmp];
	      score2 +=  CharMap[(int) *tmp];
	      special_char += SpecialCharMap[(int) *tmp]; 

             *tmp++;



	 } while (*tmp);


	/**
         * this is just bare bones of my algorythm, however we are 
         * determineing the probability that nick is a bot based on 
         * statistical averages and beysian properties.
         */

	nick_count += 1;
	nick_range_val += (float) score2;
	average = (nick_range_val / nick_count);
	
	if (avg_range == 0)
		avg_range = average;

	percent = (avg_range * 0.05); //get 5% of the system average.
	alloted_prec = avg_range + percent; //allow for a margin of percent between our current score and the average 

	/**
         *  if our current average is less then our system avg 
         *  then we dont want to update it, becuase we dont want 
	 *  giant fluctuations in our average as a lowering 
         *  of hte average will make more things look like bots.
         */

	if ((average > avg_range) && (average <= alloted_prec))
		avg_range = average;



	/**
	 * Miscelanious checks
	 * Old style hackers have a 6/5 Numeric nick + realname :)
	 * Add this filter into our beysian system;.
	 */



	 if (((strlen(u->realname) == 6) || (strlen(u->realname) == 5)) ||
	    ((strlen(u->user) == 6) || (strlen(u->user) == 5))) {

		if (score > 2)
			misc_checks = 1;
	 }


	/**
	 * Add up the basian values
	 */




	
	total = 0; //total should always = fucking 0 comming into this ;/

	if (score > 2)
		total = 1.0;

	//this check is only worth .5 so it wont tip the scale
	if (score2 > alloted_prec)
		total += 0.5;

	if (special_char >= 3)
		total += 1.0;

	if (misc_checks > 0)
		total += 0.5;

	if (cmd_check)
	{
		sendto_logchan("Scan results for %s",u->nick);

		sendto_logchan("Alpha numeric: %d [Beysian Value: %f]",score,
						(score > 2)? 1.0 : 0.0);

		sendto_logchan("Probably statistic: %f [Beysian Value: %f]",score2, 
						(score2 > alloted_prec)? 0.5 : 0.0);

		sendto_logchan("Miscelanious checks: %d [Beysian Value: %f]", misc_checks, 
						(misc_checks > 0)? 0.5 : 0.0);

		sendto_logchan("Total Score: %f",(total)? total : 0.0);
		return;
	}

	HandleResult (u, total);

    return;
}


/********************************************************/

void version_que ()
{
	User	*u;
	int		i;
	dlink_node		*dl, *tdl;
	struct scanque_	*sq;

	if (sync_state != 3)
		return;

	i = 0;

	DLINK_FOREACH_SAFE (dl, tdl, VersionQ.head)
	{
		if (i > 20)
			return;

		i++;

		sq	= dl->data;
		u	= (User *) sq->user;

		sendto_one_privmsg (s_Guardian, u, "\001VERSION\001");

		dlink_delete (dl, &VersionQ);
		dlink_free (dl);
		free (sq);
	}

	DelTimedEvent ("VersionQueue");
	return;
}


void scan_que()
{
	User* u;
	struct scanque_ *sq;
	dlink_node *dl,*tdl;
	int found = 0;
	int i = 0;

	//make this repeat untill we are synched properly
	if (sync_state != 3)
		return;
	
	if (has_q)
	{
		sendto_logchan("\002ScanQ:\002 Taking action on nicks found durring connection phase");
	
		DLINK_FOREACH_SAFE(dl,tdl,ScanQ.head)
		{
			if (i > 20) // Only do 20 nicks at a time
				return;
	
			i++;
	
			sq = dl->data;
		
			u = sq->user;
	
			if ((!u->service) || (!u->oper))
			{
				sendto_logchan("\002Scan:\002 Detected bot %s (%s@%s)",u->nick,u->user,u->host);
	
			}
	
			dl = dlink_find_delete(sq,&ScanQ);
			dlink_free(dl);
			free(sq);
		}
	}
	DelTimedEvent("SCANQ");
	return;

}

/********************************************************/

int HandleResult(User* u, int score)
{


	if (score < 2)
		return;

        alog(2,"Scan: %s recieved a score of %d",u->nick,score);
	write_bl_log(u,score);
	if (sync_state != 3)
	{
		struct scanque_ *sq;

		if (!(sq = (struct scanque_*) malloc(sizeof(struct scanque_))))
			return;

		dlink_node *dl = dlink_create();

		sq->user = u;

		dlink_add_tail(sq,dl,&ScanQ);

        has_q = 1;
	 
        return;
	}

	if (debug > 0)
		sendto_logchan("\002Debug:\002 %s (%s@%s)  recieved a scan score of %d",u->nick,u->user,u->host,score);

    sendto_logchan("\002Alert:\002 Possible spam bot detected %s (%s@%s:%s)",u->nick,u->user,u->host,u->realname);
    //send_line(":%s KILL %s :%s, You have been flagged as a possible spambot - for any question please email ADMIN_EMAIL",(s_Guardian)? s_Guardian->nick : Me->sid, u->nick, u->nick);

    return MOD_CONT;
}

/*************************************************************/

void write_bl_log(User* u, int score)
{

	fprintf(fp,"BOT: %lu %s (%s@%s:%s) %d %s (%s)\n",(unsigned long) time(NULL), u->nick,u->user,u->host,u->realname,score,u->host,u->serv->name);
	fflush(fp);
	return;	
}


/*************************************************************/
static void cmd_version   (User* u, int ac, char **av)
{



	int cnt = 0;
	char *end = NULL;
	char *space = NULL;
	char *params[512];


	if (*av[2] != '\001')
			return;

	*av[2]++;

	if ((end = strchr(av[2],'\001')))
		*end = '\0';

	if ((end = strchr(av[2],' ')))
	{
		*end++;
		if (!(space = strchr(end,' ')))
			cnt = 1;
	}
	else
		return;


       sendto_logchan("\002Version:\002 %s from %s",end, u->nick);

	if (u->version[0] == '\0')
		strlcpy (u->version, end, sizeof(u->version));


	if (cnt)
		sendto_logchan("\002Alert:\002 Possible litmus trojan detected");

       return;
}


/*******************************************************/
static void cmd_quit(User* u, int ac, char **av)
{
	//if we only increment our nick_count and never subtract from it
        //then our average will always be lower so deincrement nick_count
        //as they exit the server. Twitch
	nick_count--; 
	return; 
}



static void cmd_scanstats(User* u,char *av)
{

	int n;
	char* params[512];

	User *user;


	n = generictoken(' ',512,av,params);


	if (!av)
	{
		sendto_one(s_Guardian,u,"Scanner Statistics");
		sendto_one(s_Guardian,u,"Average: %f",avg_range);
		sendto_one(s_Guardian,u,"Total Nick Val: %f",nick_range_val);
		sendto_one(s_Guardian,u,"Nick Count: %d",nick_count);
		return;
	}

	if ((user = find_user(params[0])))
	{
		if (!(user = find_user (params[0])))
		{
			sendto_one(s_Guardian,u,"Unable to find user");
			return;
		}

		scan_nick (2, user);

		return;

	} 

	return;
}



