/*
 *          OMEGA SECURITY SERVICES
 *       (C) 2008 Omega Development Team
 *
 * Name:  module.c
 * Copyright:  2008 Omega Security Services
 * Author:  Omega Dev Team
 * Description: Module Loader and API file.
 *
 *  For information on copying please read LICENSE
 *
 *          $Id: module.c 1376 2008-09-09 07:43:31Z twitch $
 */



#include "stdinc.h"
#include "server.h"

extern int modcnt;

char *GetModErr(int);

char *ModType[] = {
        "3rd",         //0
        "Protocol",    //1
        "Core",        //2
        "Supported",   //3
        NULL
};


int module_que_count = 0;

/*****************************************/

char *GetModErr(int status)
{


	/**
	 * negative value of status is representive of module
	 * side errors, these are custom errors that modules
	 * must define.
	 */

	if (status <= 0)
	{
		switch (status)
		{
			case MOD_ERR_MSG:
				return (char *) mod_err_msg;
		}
	}


	const char *ModErr[] = {
			"Success, no error.",
			"Module Error, Module Returned STOP.",
			"Module Error, CRITICAL.",
			"Module Error, FATAL.",
			"Module Error, File already loaded.",
			"Module Error, No such file.",
			"Module Error, IO Error.",
			"Module Error, Invalid API.",
			"Module Error, Incorrect ABI version.",
			"Module Error, Allocating Memory.",
			"Module Error, Unable to unload module.",
			"Module Error, Invalid SYNTAX.",
			"Module Warning, Unsafe to unload.",
			"Module Error, Core set in no-module mode.",
			"Module Error, SQLite is not available on this system."
	};

	if (ModErr[status])
		return (char*)ModErr[status];
	else
		return (char*)"Module Warning, No return from MODULE";

}


/*****************************************/
/**
 * load_protocol() - Load our given protocol
 *                   if it fails toss a critical
 *                   error and return
 *  @return bool
 *              true  - Module has loaded successfully
 *              false - Module failed to load see logs
 *
 */


int load_protocol()
{
    int status = 0;
    char *pmod = (char*) malloc(sizeof(char) * 255);

	if (!pmod)
		return 0;

 	sprintf(pmod,"%s%s",PPATH,CfgSettings.protocol);

	if (!strstr(pmod,".so"))
		strcat(pmod,".so");

       status = module_open(pmod);

	if ((status!= 0 && status != 12))
	{
       	sendto_console("FATAL: %s \033[1;34m %s \033[0m",CfgSettings.protocol, GetModErr(status));
		alog(4,"Protocol Error: [%s] %s",pmod, GetModErr(status));
		return 0;
	}

    free(pmod);
    return 1;

}

/*****************************************/


int load_core() {
	DIR *dp;
    struct dirent *ep;

	char path[255];

    int status = 0;


	printf("Loading Core ");

	if (!(dp = opendir(CPATH)))
	{
		perror("Unable to open core module directory.");
		return 0;
	}

	while (ep = readdir (dp))
    {
		if (*ep->d_name == '.')
			continue;

            if ((ep->d_name[strlen(ep->d_name) - 3] != '.') ||
					(ep->d_name[strlen(ep->d_name) - 2] != 's')) {
				continue;
			}

		sprintf(path,"%s%s",CPATH,ep->d_name);

		status = module_open(path);

		if (status != 0)
		{
			printf("\n %s \033[1;34m %s \033[0m \n", ep->d_name, GetModErr(status));
			return 0;
		}
		printf(".");
		memset(path,0,sizeof(path));
	}

    (void) closedir (dp);

	printf(" Done\n\n");
    return 1;
}

/*****************************************/


int load_modules()
{
    int i;
    int status = 0;
    DIR *dp;
    struct dirent *ep;
    char path[255];
    char *end_curfile = NULL; //end of the current file


	if (!nomodules)
	{
		printf("\r\n");
		printf("Loading Modules\n");
		if (!(dp = opendir(MPATH)))
		{
			perror("Error");
			return 0;
		}

		while (ep = readdir (dp))
		{

			if (*ep->d_name == '.')
				continue;

			if ((ep->d_name[strlen(ep->d_name) - 3] != '.') ||
					(ep->d_name[strlen(ep->d_name) - 2] != 's')) {
				continue;
			}

			sprintf(path,"%s/%s",MODULE_DIR,ep->d_name);

			status = module_open(path);
			printf(" * Loading: \033[1;32m %s \033[0m \033[1;34m %s \033[0m \n", ep->d_name,(status != 0)? GetModErr(status) : "");
			memset(path,0,sizeof(path));
		}

		(void) closedir (dp);

		return MOD_ERR_OK;

	} else {

		printf("\r\n");
		printf("Note: Core is currently running in skeleton mode\n");
		printf("      No non-core modules will be loaded\n");
		return MOD_ERR_OK;
	}

	return MOD_ERR_OK;

}



/*****************************************/

int load_client_modules()
{
	int cnt = 0;
	int status = 0;
	char path[255];

	if (nomodules)
		return;

	//first init the guardian moodules
	while (cnt < gmod_cnt)
	{
		sprintf(path,"%s/%s",GMOD_DIR,guardian_modules[cnt]);
		status = module_open(path);
		printf(" * Loading: \033[1;32m %s \033[0m \033[1;34m %s \033[0m \n", guardian_modules[cnt],(status != 0)? GetModErr(status) : "");
		memset(path,0,sizeof(path));
		cnt++;
	}
	return 0;
}

/*****************************************/

/**
 * Module Loading/Deleteing specific globals
 */

typedef int (mod_verfnct)();

/*****************************************/

/**
  * module_open() - Load our module file symbols
  *
  * @param filename - the file we are trying to link
  *              against.
  *
  * @return int
  *          0 = MOD_ERR_OK
  *			 1 = MOD_ERR_STOP
  *			 2 = MOD_ERR_CRIT
  *			 3 = MOD_ERR_FATAL
  *			 4 = MOD_ERR_EXISTS
  *			 5 = MOD_ERR_NOFILE
  *			 6 = MOD_ERR_IO
  *			 7 = MOD_ERR_API
  *			 8 = MOD_ERR_ABI
  *			 9 = MOD_ERR_MEMORY
  *          10 = MOD_ERR_UNLOAD
  *          11 = MOD_ERR_SYNTAX
  *          12 = MOD_ERR_
  *          13 = MOD_ERR_NOMOD
  *
  *
  *  Also note we dont wanna exit if a module fails just return fail status, so
  *  the caller can handle errors.
  */


int module_open(char *filename)
{
  	Module* m;
	dlink_node *dl;
	mod_verfnct *mod_ver;
   	int rc 		= MOD_ERR_OK;
	int mod_ver_rtrn = 0;
	char		tmp[255];
	initfnct	*create_module;


	memset(mod_err_msg,0,sizeof(mod_err_msg)); //set our cust mod error to 0.
    //memset(currentModule,0,sizeof(currentModule));



	if ((!filename) || (*filename == ' '))
		return MOD_ERR_SYNTAX;


    /**
     * setup module path by first checking if its a absolute directory if it is then
     * continue on if not prefix the directory the module is found it, also check
     * to make sure that its .so :) becuase this is crucial if its not prefix it
     * if its a bunk then prefix it anyways
     */

	snprintf(tmp, sizeof(tmp), "%s%s%s", (strstr(filename,"/"))? "" : find_module_dir(filename),
		filename, (filename[strlen(filename) - 3] != '.')? ".so" : "");



    if (!module_exists(tmp))
	        return MOD_ERR_NOFILE;

    if (FindModule(tmp))
            return MOD_ERR_EXISTS;


    /**
     * Check for malloc fail
     */

    if (!(currentModule = (Module*) malloc(sizeof(Module))))
            return MOD_ERR_MEMORY;

    strlcpy(currentModule->file,tmp,sizeof(currentModule->file));

    currentModule->age = time(NULL);
    currentModule->abi    = 0;
    currentModule->type   = 0;
    currentModule->clssym = 0;

    /**
     * Open the module and run the init function.
     */

    if (!(currentModule->handle = dlopen(tmp, RTLD_NOW)))
	{
		snprintf(mod_err_msg,sizeof(mod_err_msg),"Module Error, %s",dlerror());
		alog(2,"Error when loading module %s [%s]",tmp,mod_err_msg);
		return MOD_ERR_MSG;
	}


	if (!(create_module = (initfnct*) dlsym(currentModule->handle,"Module_Init")))
	{
		snprintf(mod_err_msg,sizeof(mod_err_msg),"Module Error, %s",dlerror());
		alog(2,"Error when loading module %s [%s]",tmp,mod_err_msg);
		return MOD_ERR_MSG;
	}

	currentModule->close = (closefnct*) dlsym(currentModule->handle,"Module_Close");

	if (currentModule->close)
		currentModule->clssym = 1;


	if ((rc = create_module()) != 0)
	{
		module_free(currentModule);
		return rc;

	}


    /**
     * Module ABI Checks
     */


	int chk_ver = ABI_VERSION - ABI_PATCH;

	if (currentModule->abi != ABI_VERSION)
	{
		if (currentModule->abi >= (chk_ver + 10))
		{
                module_free(currentModule);
				return MOD_ERR_ABI;
		}
		if (currentModule->abi <= (chk_ver - 10))
		{
                module_free(currentModule);
				return MOD_ERR_ABI;
		}
	}

	dl = dlink_create();
	dlink_add_tail (currentModule, dl, &modules);

	alog(10,"Loaded Module: %s",filename);

	return MOD_ERR_OK;
}

/************************************************
 *               MODULE FIND                    *
 ************************************************/

Module* FindModule(char *module)
{
      dlink_node* dl;
      Module* m;

    if (!module)
        return NULL;

	DLINK_FOREACH (dl, modules.head)
	{
		m = dl->data;
		if ((strcasecmp(m->name,module)==0) || (strcasecmp(m->file,module)==0))
		                return m;

    }

    return NULL;
}

/************************************************
 *                 MODULE API                   *
 ************************************************/


/**
 * RegisterModule - Register module with the core.
 */


int RegisterModule(char* name,char* author,char *version, int major, int minor, int patch)
{

       /**
        * add some logic here our ABI should look like this
        *  [0][0][00]
        *  ^   ^  ^^
        *  |   |   |
        *  `---|---|---------- MAJOR
        *      `---|---------MINOR
        *          `-----PATCH
        *
        *   Note as we progress the core will generate its ABI
        *   using a similar method that way ABI will either be
        *   greater than, equal, or less then and will save us
        *   from having to painstakenly check each variable.
        */


        currentModule->abi = ((major * 1000) + (minor  * 100) + patch);

		strlcpy(currentModule->name,name,sizeof(currentModule->name));
		strlcpy(currentModule->version,version,sizeof(currentModule->version));
		strlcpy(currentModule->author,author,sizeof(currentModule->author));

	return MOD_CONT;
}

/*********************************************************/

int CheckVer()
{
	int chk_ver = ABI_VERSION - ABI_PATCH;

	if (currentModule->abi != ABI_VERSION)
	{
		if (currentModule->abi >= (chk_ver + 10))
				return 0;

		if (currentModule->abi <= (chk_ver - 10))
				return 0;

	}
	return 1;
}


/*********************************************************/


void module_free(Module *m)
{
	assert (m != NULL);
	MyFree(m);
}

/*********************************************************/

char *GetType(Module *m)
{
	return ModType[m->type];
}

/*********************************************************/
/**
 * Delete a module from the system
 * @return
 *       0 - success
 *       1 - no such module
 *       2 - Invalid Type
 */

int module_close (char *module)
{

	dlink_node* dl;
	char path[255];

	if (!strstr(module,"/"))
		sprintf(path,"%s/%s%s",find_module_dir(module),module,
					(module[strlen(module) - 3] != '.')? ".so" : "");
	else
		sprintf(path,"%s%s",module,
				(module[strlen(module) - 3] != '.')? ".so" : "");


	alog(DEBUG3,"Trying to unload module (%s) (%s)",module,path);

	DLINK_FOREACH (dl, modules.head)
	{
		currentModule = dl->data; //instead of using m blah and assigning current module there just
					  //just use current module and work from there :)

		if (strcasecmp(currentModule->name,module)==0)
                {

			alog(10,"Deleting Module %s",module);

			if (currentModule->clssym)
				currentModule->close();

			if (currentModule->handle)
				dlclose(currentModule->handle);

			dl = dlink_find_delete (currentModule, &modules);
			dlink_free (dl);

			module_free(currentModule); //new function added here free all info within currentModule before freeing current 			   	    //module - assert to trap bad params passing to it.

			return MOD_ERR_OK;
		}
	}
	return MOD_ERR_UNLOAD;
}

/**********************************************************/
/**
 *   ModuleQ - Que module operations so that we dont
 *   unload modules while they are active...
 *
 *   @param modname - The name of the module that we are
 *                    queing up.
 *   @param op      - The action to take when module q
 *                    is ran.
 *
 *   return MOD_ERR_*
 *
 */


int addModuleQue(char *modname, int op)
{

	dlink_node* dl = dlink_create ();
	ModuleQEntry *entry;

	if ((op == 1) && (!(FindModule(modname))))
		return MOD_ERR_NOFILE;

	entry = (ModuleQEntry*) malloc(sizeof(ModuleQEntry));

	entry->name = strdup(modname);
	entry->action = op;

	dlink_add_tail(entry,dl,&moduleque);

	alog(DEBUG3,"Adding %s to module que (Action %d)",modname,op);

	return MOD_ERR_OK;
}


int runModuleQue()
{
	dlink_node *dl,*tdl;
	ModuleQEntry *entry;
	int found = 0;
	int status = 0;

	DLINK_FOREACH_SAFE(dl, tdl, moduleque.head)
	{
			entry = dl->data;

			alog(DEBUG3,"Running action %d on module %s",entry->action,entry->name);

			if (entry->action)
			{
				status = DelModule(entry->name);
				sendto_logchan("\002Module:\002 Unloading module [%s] - %s", entry->name, GetModErr(status));

			} else {
				status = module_open(entry->name);
				sendto_logchan("\002Module:\002 Loading module [%s] - %s", entry->name, GetModErr(status));

			}

		free(entry->name);
		dl = dlink_find_delete(entry,&moduleque);
		dlink_free(dl);
		free(entry);

	}

	return 0;
}


/**********************************************************/
/**
 * Unload all modules regradless of flavor
 */

int unload_all_modules(int unload_protocol)
{
       dlink_node *dl = NULL;
       dlink_node *tdl = NULL;
       Module *m = NULL;

	DLINK_FOREACH_SAFE(dl,tdl,modules.head)
	{

		currentModule = dl->data;

		if ((currentModule->type == 1) && (!unload_protocol))
			continue;

		if (currentModule->clssym)
			currentModule->close();
		/**
		 * Dont try to close modules that FAILED to open
		 * and produce a proper handle, this happens when
		 * modules have undefined referances... and are
		 * bunk all together. Twitch
		 */

		if (currentModule->handle)
			dlclose(currentModule->handle);

         dl = dlink_find_delete(currentModule,&modules);
		 dlink_free(dl);

		 module_free(currentModule);
	}
	return 0;
}


/**********************************************************/

int module_exists (char * fileName)
{
   struct stat buf;
   char file[255];

	int	i;

   snprintf(file,sizeof(file),"%s",fileName);

   if ((fileName[strlen(fileName) - 3] != '.') ||
		(fileName[strlen(fileName) - 2] != 's'))
	strlcat(file,".so",sizeof(file));

   i = stat ( file, &buf );
   memset(file,0,sizeof(file));

   return (i == 0)? 1 : 0;

}

/**********************************************************/


char *find_module_dir(char *module)
{
	char path[255];
	//first look in mpath
	sprintf(path,"%s/%s",MPATH,module);

	if (module_exists(path))
		return MPATH;

	memset(path,0,sizeof(path));

	sprintf(path,"%s/%s",PPATH,module);

	if (module_exists(path))
		return MPATH;

	memset(path,0,sizeof(path));
	
	sprintf(path,"%s/%s",CPATH,module);

	if (module_exists(path))
		return CPATH;

	memset(path,0,sizeof(path));

	sprintf(path,"%s/%s",GMOD_DIR,module);

	if (module_exists(path))
		return GMOD_DIR;

	memset(path,0,sizeof(path));

	sprintf(path,"%s/%s",CONTRIB_PATH,module);

	if (module_exists(path))
		return CONTRIB_PATH;

	memset(path,0,sizeof(path));

	return MPATH; //return MPATH no matter what as our module_open will just shoot out module doesnt exsists

}

/**************************************************************/


int rehash_client_modules()
{

	Module *m;
	int cnt = 0;
	char path[255];
	int status = 0;

	for (cnt = 0; cnt < gmod_cnt; cnt++)
	{
		memset(path,0,sizeof(path));
		sprintf(path,"%s/%s",GMOD_DIR,guardian_modules[cnt]);

		if ((m =  FindModule(path)))
			continue;

		status = module_open(path);
		sendto_logchan("\002Module:\002 Loading module %s - %s",guardian_modules[cnt],GetModErr(status));
	}
	return 0;
}


/*****************************************************************/


int rehash_core_modules()
{

	DIR *dp;
    struct dirent *ep;
	char path[255];
	Module* m;

	int status = 0;

	if (!(dp = opendir(CPATH)))
	{
		sendto_logchan("\002Rehash:\002 Unable to open core module directory [%s].",CPATH);
		return 1;
	}

	while (ep = readdir (dp))
       {
		m = NULL;

		if (*ep->d_name == '.')
			continue;

		sprintf(path,"%s%s",CPATH,ep->d_name);

		if ((m = FindModule(path)))
			continue;

		status = module_open(path);
		sendto_logchan("Module: Loading module %s - %s", ep->d_name, GetModErr(status));

	}

	(void) closedir (dp);



	if (!(dp = opendir(MODULE_DIR)))
	{
		sendto_logchan("\002Rehash:\002 Unable to open core module directory [%s].",MPATH);
		return 1;
	}


	while (ep = readdir (dp))
       {
		m = NULL;

		if (*ep->d_name == '.')
			continue;

            if (ep->d_name[strlen(ep->d_name) - 3] != '.')
                    continue;

            sprintf(path,"%s/%s",MODULE_DIR,ep->d_name);

		if ((m = FindModule(path)))
			continue;

		status = module_open(path);
		sendto_logchan("Module: Loading module %s - %s", ep->d_name, GetModErr(status));

	}

       (void) closedir (dp);

	return 0;

}



/*****************************************************************/






