#include "stdinc.h"
#include "server.h"

//server 0 is the uplink server were as
//it will have 2 spots in the link
int servernum = 0;

extern Link* serv_list;

IRCDProto IRCd;


int protocol_init()
{


     /*
	  ** lets start with a clean struct so if something
	  ** is null we know its not supported :) Twitch
	  */

	IRCd.ircd_connect	= ircd_null;
	IRCd.ircd_ping		= ircd_null;
	IRCd.ircd_add_user	= ircd_null;
	IRCd.ircd_oper		= ircd_null;
	IRCd.ircd_wallop	= ircd_null;
	IRCd.ircd_xline		= ircd_null;
	IRCd.ircd_version	= ircd_null;
	IRCd.ircd_set_cmode	= ircd_null;
	IRCd.ircd_join		= ircd_null;
	IRCd.ircd_mode 		= ircd_null;
	IRCd.ircd_umode 	= ircd_null;
	IRCd.ircd_kill		= ircd_null;

}



/*******************************************/


void add_protocol(char* name,char *version,char *author, int ts6)
{
	memcpy(IRCd.name,name,sizeof(IRCd.name));
	memcpy(IRCd.version,version,sizeof(IRCd.version));
	memcpy(IRCd.author,author,sizeof(IRCd.author));
	IRCd.age = time(NULL);

    if (ts6 > 0)
        IRCd.ts = 1;

//	IRCd.ts6 = ts6;

	alog(0,"Loaded Protocol: \033[1;32m%s\033[0m",name);

	if (!debug ||!nofork)
		printf("Loaded Protocol: \033[1;32m%s\033[0m\n",name);

	if (ts6 > 0)
	{
		alog(0,"Enabled TS6 Support\n");
		if (!debug ||!nofork)
			printf("Enabled TS6 Support\n");
	}
}

void addp_connect(void (*func) ())
{
     IRCd.ircd_connect = func;
}

void addp_oper(void (*func)(char *source, char *opertype))
{
	IRCd.ircd_oper = func;
}

void addp_add_user(void (*func)(User* user))
{
	IRCd.ircd_add_user = func;
}

void addp_ping(void (*func)(char *source))
{
 	IRCd.ircd_ping = func;
}

void addp_wallop(void (*func)(char *source,char *msg))
{
	IRCd.ircd_wallop = func;
}

void addp_xline(void (*func)(char *xline,char *source,char *mask, char *reason))
{
	IRCd.ircd_xline = func;
}

void addp_version(void (*func)(User* user))
{
	IRCd.ircd_version = func;
}

void addp_set_cmode(void (*func)(Channel*,int,char *))
{
	IRCd.ircd_set_cmode = func;
}

void addp_set_umode(void (*func)(User*,int,char *))
{
	IRCd.ircd_set_umode = func;
}

void addp_join(void (*func)(User*,Channel*))
{
	IRCd.ircd_join = func;
}

void addp_mode(void (*func)(User*,Channel*,char*))
{
        IRCd.ircd_mode = func;
}

void addp_umode(void (*func)(User*,char*))
{
        IRCd.ircd_umode = func;
}

void addp_kill (void (*func)(User *, char *))
{
	IRCd.ircd_kill	= func;
}

/*******************************************/

void ircd_mode(User* u,Channel* c,char *mode)
{

        IRCd.ircd_mode(u,c,mode);
}

/*******************************************/

void ircd_umode(User* u,char *mode)
{

        IRCd.ircd_umode(u,mode);
}


/*******************************************/

/* Assign this to avoid cores when functions are not defined in the protocol */
void ircd_null()
{
	return;
}

void ircd_xline(char *xline, char *source, char *mask, char *reason)
{

	IRCd.ircd_xline(xline,source,mask,reason);
}

void ircd_version (User* user)
{
	IRCd.ircd_version (user);
}

void ircd_connect()
{
    IRCd.ircd_connect();
}

/*void ircd_add_user(char *source,char *ident, char *host,char *modes, char *name)*/
void ircd_add_user(User* user)
{
    IRCd.ircd_add_user(user);
}

void ircd_ping(char *source)
{
	IRCd.ircd_ping(source);
}

void ircd_oper(char *source, char *opertype)
{
	IRCd.ircd_oper(source,opertype);
}

void ircd_wallop(char *source,char *msg, ...)
{
        char *tmp = (char *) malloc(sizeof(char) * 512);
	va_list args;

	va_start(args,msg);
	vsnprintf(tmp,512,msg,args);

	if (source == NULL)
		IRCd.ircd_wallop (CfgSettings.servername, tmp);
	else
        IRCd.ircd_wallop (source, tmp);

        va_end(args);
	free(tmp);
}

void ircd_set_cmode(Channel* u,int argc,char *modes)
{
	IRCd.ircd_set_cmode(u,argc,modes);
}

void ircd_set_umode(User* u,int argc,char *modes)
{
	IRCd.ircd_set_umode(u,argc,modes);
}


void ircd_join(User* u,Channel* c)
{
	IRCd.ircd_join(u,c);
}

void ircd_kill (User *u, char *message)
{
	IRCd.ircd_kill (u, message);
}

/*****************************************/

Link	*new_serv (Link *uplink, char *servername)
{
	Link		*tmp;
	dlink_node	*dl;

	if ((tmp = find_serv (servername)))
		return tmp;

	tmp	= (Link*) malloc(sizeof(Link));
	dl	= dlink_create ();

	memcounts.servsize	+= sizeof(tmp);
	memcounts.servcnt++;

	if (!valid_servername(servername))
		Log("Invalid servername from %s",servername);

	strlcpy (tmp->name, servername, sizeof(tmp->name));

	tmp->linktime	= time(NULL);
	tmp->eos		= 0;
	tmp->uplink		= (uplink)? uplink : NULL;
	tmp->sid[0]		= '\0';

	dlink_add_tail (tmp, dl, &servlist);

	servernum++;

	sendto_logchan ("\002Server Connect:\002 %s connecting to %s", servername, (uplink)? uplink->name : "<No Uplink>");

	return tmp;
}

/*****************************************/

Link* find_serv(char *serv)
{
    Link        *li;
    dlink_node  *dl;

    if (!serv)
        return NULL;

    DLINK_FOREACH (dl, servlist.head)
    {
        li  = dl->data;

        if (!li->name)
            continue;

        if ((strcasecmp (li->name, serv)) == 0)
            return li;

        if (li->sid[0] != '\0')
            if ((strcasecmp (li->sid, serv)) == 0)
                return li;
    }

	return NULL;
}

/*****************************************/
void exit_serv (char *serv)
{
    Link        *li, *s;
    User        *u;
    dlink_node  *dl, *tdl;

    if (!serv)
        return;

    if (!(s = find_serv (serv)))
        return;

    DLINK_FOREACH_SAFE (dl, tdl, servlist.head)
    {
        li  = dl->data;

        if (li->uplink)
        {
            if ((strcasecmp (li->uplink->name, s->name)) == 0)
                exit_serv (li->name);
        }
    }

    sendto_logchan ("\002Netsplit:\002 %s split from %s", s->name, s->uplink->name);

    /* Exit users that are on the server we are exiting */

    DLINK_FOREACH_SAFE (dl, tdl, userlist.head)
    {
        u   = dl->data;

        if ((u) && (u->serv))
            if ((strcasecmp(s->name, u->serv->name)) == 0)
                exit_one_user(u, NULL);
    }

    dl = dlink_find_delete (s, &servlist);
    dlink_free (dl);

	memcounts.servsize	-= sizeof(s);
	memcounts.servcnt--;
    free (s);

    return;

}

void exit_all_servs (void)
{
	Link		*li;
	dlink_node	*dl, *tdl;

	DLINK_FOREACH_SAFE (dl, tdl, servlist.head)
	{
		li	= dl->data;

		if ((strcasecmp (li->name, CfgSettings.servername)) == 0)
		{
			li->uplink	= NULL;
			continue;
		}

		dlink_delete (dl, &servlist);
		dlink_free (dl);

		free (li);
	}

	return;
}

/***************************************/

void  update_eos(char *serv)
{
	Link* tmp = find_serv(serv);

	if (!tmp)
		return;
	else
		tmp->eos = 1;

	return;
}

/*******************************************/

int link_count(void)
{
	return servernum;
}
