

#include "stdinc.h"
#include "server.h"


int has_dest_threads = 0;


Thread* spawn_thread(void *params, void (*init_fnct)(void *),void (*exit_fnct)(void *))
{

#ifdef    HAVE_PTHREAD

    Thread *t;
    dlink_node *dl;

    int status = 0;
    int thread_current = 0;

    if (!thread_count)
        thread_count = 0;

    t = (Thread*) malloc(sizeof(Thread));

    pthread_mutex_init(&t->lock, NULL); //initialize the mutext

    pthread_attr_init(&t->attr);
    pthread_attr_setdetachstate(&t->attr, PTHREAD_CREATE_JOINABLE);

    pthread_cond_init (&t->cond, NULL);



    switch ((status = pthread_create(&t->tid, &t->attr, (void*) init_fnct, params)))
    {
                case 0:
                    thread_count++; //only increment our thread_count if we successfully spawn :)
                    break;

                case EAGAIN:
                        alog(ERROR,"Unable to spawn threaded process - nessicary reasources locked");
                        return NULL;
                case EINVAL:
                        alog(ERROR,"Unable to spawn threaded process - invalid attributes");
                        return NULL;
                case EPERM:
                        alog(ERROR,"Unable to spawn threaded process - incorrect permissions");
                        return NULL;
    }

    t->params = params;
    t->cleanup = exit_fnct;


    dl = dlink_create();
    dlink_add_tail(t,dl,&threadlist);

    //detach from the thread.
    pthread_detach(t->tid);

    return t;

#endif /* HAVE_PTHREAD */

    return NULL;
}



int mutex_state(Thread* thread, int lock)
{


    s_assert(&thread != NULL);
    s_assert(lock >= 0);

    if (!thread)
		return 0;

#ifdef HAVE_PTHREAD

       switch (lock)
       {

            case THREAD_MUTEX_LOCK:
                    pthread_mutex_trylock(&thread->lock);
                    break;
            case THREAD_MUTEX_UNLOCK:
                    pthread_mutex_unlock(&thread->lock);
                    break;

        }

#endif

    return 1;
}



Thread *find_thread()
{
#ifdef HAVE_PTHREAD

	dlink_node *dl,*tdl;
	Thread *thread;

	DLINK_FOREACH_SAFE(dl,tdl,threadlist.head)
	{
		thread = dl->data;

		if (pthread_equal(pthread_self(), thread->tid))
			return thread;
	}
#endif
	return NULL;

}


int dest_thread()
{

#ifdef HAVE_PTHREAD
	dlink_node *dl,*tdl;
	Thread *thread;

	DLINK_FOREACH_SAFE(dl,tdl,threadlist.head)
	{
		thread = dl->data;

		if (!pthread_equal(pthread_self(), thread->tid))
			continue;


		if (thread->cleanup)
		{
			pthread_cleanup_push(thread->cleanup, thread->params);
			pthread_cleanup_pop(1);
		}

		dl = dlink_find_delete(thread,&threadlist);
		dlink_free(dl);

	}

	if (thread_count > 0)
		thread_count--;

	pthread_exit((void*) NULL);
#endif
	return 1;

}



void dest_all_threads(void)
{

#ifdef HAVE_PTHREAD

	dlink_node *dl,*tdl;
	Thread *thread;
	int thread_total = 0;
	int thread_status = 0;

	if (!has_dest_threads)
        has_dest_threads = 1;
    else
        return;


	DLINK_FOREACH_SAFE(dl,tdl,threadlist.head)
	{
		thread = dl->data;

		if (thread->cleanup)
			thread->cleanup(thread->params);


        thread_status = pthread_cancel(thread->tid);

		switch (thread_status)
		{
			case 0:
				break;
			case ESRCH:
				break;
			default:
				Log("Thread Error, Unable to destroy thread unknown return in pthread_cancel");
				break;
		}

		if (thread_total > 0)
                thread_total--;

		dl = dlink_find_delete(thread,&threadlist);
		dlink_free(dl);
		free(thread);

	}

	alog(2,"Destroyed %d thread(s)",thread_count);
#endif

	return;
}

